local myname="ZifnabsUI"
local Me=CreateFrame("Frame",myname)
if not ZifnabsMod then ZifnabsMod={ui=Me} else ZifnabsMod.ui=Me end
local mod
local L
local db
local mainframe

-- --------------------------------------------------------------------------------------
-- INITIALIZE
-- --------------------------------------------------------------------------------------
function Me:Initialize(fwinfo)
	mod=fwinfo.mod
	L=fwinfo.L
	db=fwinfo.db
end

-- --------------------------------------------------------------------------------------
-- NEW WIDGET
-- --------------------------------------------------------------------------------------
function Me:NewWidget(name,objecttype,sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,inherit,transparent,hide)
	local parentobject=getglobal(parent) or parent or UIParent
	local widget=CreateFrame(objecttype,name,parentobject,inherit)
	if transparent then widget:SetBackdrop(nil) end
	if sx and sy then widget:SetSize(sx,sy) end
	if anchorx and anchory then 
		widget:SetPoint(anchorparentpoint,parentobject:GetName(),anchorselfpoint,anchorx,anchory) 
	end
	if db.ui and db.ui[name] then
		local anchor=db.ui[name]
		widget:SetPoint(anchor.point,anchor.parent or UIParent,anchor.parentpoint,anchor.xo,anchor.yo)
	end
	if hide then widget:Hide() end
	return widget
end

-- --------------------------------------------------------------------------------------
-- NEW FRAME
-- --------------------------------------------------------------------------------------
function Me:newFrame(name,sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,opt)
	if not opt then opt={} end
	local frame=Me:NewWidget(name,"Frame",sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,opt.inherit or "ZifnabsMovableFrame",opt.transparent,opt.hide)
	if opt.mainframe then mainframe=frame end
	return frame
end

-- --------------------------------------------------------------------------------------
-- NEW BUTTON
-- --------------------------------------------------------------------------------------
function Me:newButton(name,sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,opt)
	if not opt then opt={} end
	local button=Me:NewWidget(name,"Button",sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,opt.inherit or "ZifnabsButton",opt.transparent,opt.hide)
	if opt.text then button:SetText(opt.text) end
	if opt.click then button.click=opt.click end
	if opt.tt then button.tooltipText=opt.tt end
	return button
end

function Me:newControlButton(slot,name,opt)
	local anchorx=-14-16*slot
	local anchory=6
	if slot>3 then
		local row=math.floor((slot+1)/5)
		local col=slot+1-5*row
		anchorx=-62+col*16
		anchory=6-16*row
	end
	while anchorx<-62 do
		anchorx=anchorx+62
		anchory=anchory-16
	end
	return Me:newButton(name,20,16,mainframe,"BOTTOMRIGHT","TOPRIGHT",anchorx,anchory,opt)
end

-- --------------------------------------------------------------------------------------
-- NEW EDITBOX
-- --------------------------------------------------------------------------------------
function Me:newEditBox(name,sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,opt)
	if not opt then opt={} end
	local editbox=Me:NewWidget(name,"EditBox",sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,"ZifnabsEditBox",opt.transparent,opt.hide)
	editbox:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = nil})
	editbox:SetBackdropColor(0,0,0,1)
	editbox.processkeys=opt.onkey
	editbox:SetScript("OnKeyDown",Me.EditBoxKeyDown)
	return editbox
end
function Me:EditBoxEnterPressed(box,arg2,arg3)
	if type(box)=="table" then for key,value in pairs(box) do mod.f:Print("key "..key) end else mod.f:Print("box="..(box or "nil")) end
	if type(arg2)=="table" then for key,value in pairs(arg2) do mod.f:Print("arg2 "..key) end else mod.f:Print("arg2="..(arg2 or "nil")) end
	if type(arg3)=="table" then for key,value in pairs(arg3) do mod.f:Print("arg3 "..key) end else mod.f:Print("arg3="..(arg3 or "nil")) end
end
function Me:EditBoxKeyDown(key)
	if self.processkeys then self.processkeys(dummy,self,key) else mod.f:Print("no handler for key "..key) end
end

-- --------------------------------------------------------------------------------------
-- ADD TEXT
-- --------------------------------------------------------------------------------------
function Me:addText(name,sx,sy,parent,anchorselfpoint,anchorparentpoint,anchorx,anchory,fontsize,justifyh,justifyv)
	local parentobject=getglobal(parent) or parent or UIParent
	local widget=parent:CreateFontString()
	if sx and sy then widget:SetSize(sx,sy) end
	if anchorx and anchory then widget:SetPoint(anchorparentpoint,parentobject:GetName(),anchorselfpoint,anchorx,anchory) end
	widget:SetFontObject("GameFontNormal")
	if fontsize then 
		local font,height,flags=widget:GetFont()
		widget:SetFont(font,fontsize)
	end
	if justifyh then widget:SetJustifyH(justifyh) end
	if justifyv then widget:SetJustifyV(justifyv) end
	return widget
end

-- --------------------------------------------------------------------------------------
-- BUTTON CLICK
-- --------------------------------------------------------------------------------------
function Me:ButtonClick(button)
	if button.click then 
		local parse=mod.f:Split(button:GetName(),"_")
		button.click(doesnt_go_to_method,button,parse) 
	end
end

-- --------------------------------------------------------------------------------------
-- TOOLTIP
-- --------------------------------------------------------------------------------------
function Me:Tooltip(widget)
	if widget.tooltipText then
		GameTooltip:SetOwner(widget, "ANCHOR_RIGHT")
		local distance=""
		if widget.distance2 then 
			local dist=mod.loc:Distance(widget.distance2)
			if dist<99999 then distance="\n"..L.word.distance.." "..format("%.2f",dist) end
		end
		GameTooltip:SetText(widget.tooltipText..distance, 1.0, 0.8, 0.0, 1 )
		GameTooltip:Show();
	end
end

-- --------------------------------------------------------------------------------------
-- MOVED
-- --------------------------------------------------------------------------------------
function Me:Moved(frame)
	local name=frame:GetName()
	point, parent, parentpoint, xo, yo = frame:GetPoint()
	if not db.ui then db.ui={} end
	db.ui[name]={point=point,parent=parent,parentpoint=parentpoint,xo=xo,yo=yo}
end